@@ -2,21 +2,19 @@ package commands
 
 import (
 	"cmp"
+	"context"
 	"encoding/json"
 	"errors"
 	"fmt"
 	"os"
 	"regexp"
+	"runtime"
 	"strconv"
 	"strings"
 	"sync"
 	"sync/atomic"
 	"time"
-
-	"github.com/nozzle/throttler"
-	"github.com/urfave/cli/v2"
-	"golang.org/x/exp/slices"
-	"golang.org/x/net/idna"
+	"unsafe"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/bindserial"
@@ -29,6 +27,11 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/pkg/providers"
 	"github.com/StackExchange/dnscontrol/v4/pkg/rfc4183"
 	"github.com/StackExchange/dnscontrol/v4/pkg/zonerecs"
+	"github.com/dustin/go-humanize"
+	"github.com/nozzle/throttler"
+	"github.com/urfave/cli/v3"
+	"golang.org/x/exp/slices"
+	"golang.org/x/net/idna"
 )
 
 type cmdZoneCache struct {
@@ -41,7 +44,7 @@ var _ = cmd(catMain, func() *cli.Command {
 	return &cli.Command{
 		Name:  "preview",
 		Usage: "read live configuration and identify changes to be made, without applying them",
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, c *cli.Command) error {
 			return exit(PPreview(args))
 		},
 		Flags: args.flags(),
@@ -90,7 +93,7 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 		Destination: &args.ConcurMode,
 		Value:       "concurrent",
 		Usage:       `Which providers to run concurrently: concurrent, none, all`,
-		Action: func(c *cli.Context, s string) error {
+		Action: func(ctx context.Context, c *cli.Command, s string) error {
 			if !slices.Contains([]string{"concurrent", "none", "all"}, s) {
 				fmt.Printf("%q is not a valid option for --cmode.  Values are: concurrent, none, all\n", s)
 				os.Exit(1)
@@ -103,7 +106,7 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 		Destination: &args.ConcurMax,
 		Value:       999,
 		Usage:       `Maximum number of concurrent connections`,
-		Action: func(c *cli.Context, v int) error {
+		Action: func(ctx context.Context, c *cli.Command, v int) error {
 			if v < 1 {
 				fmt.Printf("%d is not a valid value for --cmax.  Values must be 1 or greater\n", v)
 				os.Exit(1)
@@ -131,7 +134,7 @@ func (args *PPreviewArgs) flags() []cli.Flag {
 		Name:   "reportmax",
 		Hidden: false,
 		Usage:  `Limit the IGNORE/NO_PURGE report to this many lines (Expermental. Will change in the future.)`,
-		Action: func(ctx *cli.Context, maxreport int) error {
+		Action: func(ctx context.Context, c *cli.Command, maxreport int) error {
 			printer.MaxReport = maxreport
 			return nil
 		},
@@ -154,7 +157,7 @@ var _ = cmd(catMain, func() *cli.Command {
 	return &cli.Command{
 		Name:  "push",
 		Usage: "identify changes to be made, and perform them",
-		Action: func(ctx *cli.Context) error {
+		Action: func(ctx context.Context, c *cli.Command) error {
 			return exit(PPush(args))
 		},
 		Flags: args.flags(),
@@ -409,8 +412,10 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 		fmt.Fprintf(os.Stderr, "##teamcity[buildStatus status='SUCCESS' text='%d corrections']", totalCorrections)
 	}
 	rfc4183.PrintWarning()
+	out.PrintfIf(fullMode, "Inaccurate statistics: %s\n", stats(cfg))
 	notifier.Done()
 	out.Printf("Done. %d corrections.\n", totalCorrections)
+
 	err = writeReport(report, reportItems)
 	if err != nil {
 		return errors.New("could not write report")
@@ -424,6 +429,56 @@ func prun(args PPreviewArgs, push bool, interactive bool, out printer.CLI, repor
 	return nil
 }
 
+// stats returns a JSON string with memory usage statistics.
+// These stats are unofficial and subject to change without notice.
+// "average_mem_per_record" is misleading because it includes all memory overhead.
+func stats(cfg *models.DNSConfig) string {
+
+	// https://www.datadoghq.com/blog/go-memory-metrics/
+	// [T]he following expression accurately reflects the value the runtime attempts to maintain as the limit:
+	// runtime.MemStats.Sys − runtime.MemStats.HeapReleased
+	runtime.GC()
+	var m runtime.MemStats
+	runtime.ReadMemStats(&m)
+	memoryInUse := m.Sys - m.HeapReleased
+
+	numRecords := countRecords(cfg)
+	memPerRecord := int64(float64(memoryInUse) / float64(max(1, numRecords)))
+	memPerRecordStr := humanize.IBytes(uint64(memPerRecord)) + " bytes"
+
+	statsInfo := struct {
+		MemoryInUse    uint64 `json:"memory_in_use"`
+		MemoryInUseStr string `json:"memory_in_use_str"`
+		NumRecords     int    `json:"num_records"`
+		NumZones       int    `json:"num_zones"`
+		RCSize         int    `json:"rc_size"`
+		Benchmark1     int64  `json:"benchmark1"`
+		Benchmark1Str  string `json:"benchmark1str"`
+	}{
+		MemoryInUse:    memoryInUse,
+		MemoryInUseStr: humanize.Bytes(memoryInUse),
+		NumRecords:     numRecords,
+		NumZones:       len(cfg.Domains),
+		RCSize:         int(unsafe.Sizeof((models.RecordConfig{}))),
+		Benchmark1:     memPerRecord,
+		Benchmark1Str:  memPerRecordStr,
+	}
+
+	jsonBytes, err := json.Marshal(statsInfo)
+	if err != nil {
+		return fmt.Sprintf("error marshaling stats: %v", err)
+	}
+	return string(jsonBytes)
+}
+
+func countRecords(cfg *models.DNSConfig) int {
+	total := 0
+	for _, domain := range cfg.Domains {
+		total += len(domain.Records)
+	}
+	return total
+}
+
 // whichZonesToProcess takes a list of DomainConfigs and a filter string and
 // returns a list of DomainConfigs whose Domain.UniqueName matched the
 // filter. The filter string is a comma-separated list of domain names. If the
