@@ -5,7 +5,7 @@ import (
 	"encoding/json"
 	"errors"
 	"fmt"
-	"net"
+	"net/netip"
 	"os"
 	"strconv"
 	"strings"
@@ -199,8 +199,17 @@ func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 
 	var corrections []*models.Correction
 
+	// Check if comment/tag management is enabled for this domain
+	manageComments := dc.Metadata[metaManageComments] == "true"
+	manageTags := dc.Metadata[metaManageTags] == "true"
+
+	// Create a comparable function that includes comments/tags only if management is enabled
+	comparableFunc := func(rec *models.RecordConfig) string {
+		return genComparableWithMgmt(rec, manageComments, manageTags)
+	}
+
 	// Cloudflare is a "ByRecord" API.
-	instructions, actualChangeCount, err := diff2.ByRecord(records, dc, genComparable)
+	instructions, actualChangeCount, err := diff2.ByRecord(records, dc, comparableFunc)
 	if err != nil {
 		return nil, 0, err
 	}
@@ -256,7 +265,8 @@ func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 	return corrections, actualChangeCount, nil
 }
 
-func genComparable(rec *models.RecordConfig) string {
+func genComparableWithMgmt(rec *models.RecordConfig, manageComments, manageTags bool) string {
+	var parts []string
 	if rec.Type == "A" || rec.Type == "AAAA" || rec.Type == "CNAME" {
 		proxy := rec.Metadata[metaProxy]
 		if proxy != "" {
@@ -266,10 +276,28 @@ func genComparable(rec *models.RecordConfig) string {
 			if proxy == "off" {
 				proxy = "false"
 			}
-			return "proxy=" + proxy
+			parts = append(parts, "proxy="+proxy)
 		}
 	}
-	return ""
+	if rec.Type == "CNAME" {
+		flatten := rec.Metadata[metaCNAMEFlatten]
+		if flatten == "on" {
+			parts = append(parts, "flatten=true")
+		} else {
+			parts = append(parts, "flatten=false")
+		}
+	}
+	// Include comment in comparison only if comment management is enabled
+	if manageComments {
+		comment := rec.Metadata[metaComment]
+		parts = append(parts, "comment="+comment)
+	}
+	// Include tags in comparison only if tag management is enabled
+	if manageTags {
+		tags := rec.Metadata[metaTags]
+		parts = append(parts, "tags="+tags)
+	}
+	return strings.Join(parts, ",")
 }
 
 func (c *cloudflareProvider) mkCreateCorrection(newrec *models.RecordConfig, domainID, msg string) []*models.Correction {
@@ -404,10 +432,15 @@ func (c *cloudflareProvider) checkUniversalSSL(dc *models.DomainConfig, id strin
 }
 
 const (
-	metaProxy        = "cloudflare_proxy"
-	metaProxyDefault = metaProxy + "_default"
-	metaOriginalIP   = "original_ip" // TODO(tlim): Unclear what this means.
-	metaUniversalSSL = "cloudflare_universalssl"
+	metaProxy          = "cloudflare_proxy"
+	metaProxyDefault   = metaProxy + "_default"
+	metaOriginalIP     = "original_ip" // TODO(tlim): Unclear what this means.
+	metaUniversalSSL   = "cloudflare_universalssl"
+	metaCNAMEFlatten   = "cloudflare_cname_flatten"
+	metaComment        = "cloudflare_comment"
+	metaTags           = "cloudflare_tags"
+	metaManageComments = "cloudflare_manage_comments"
+	metaManageTags     = "cloudflare_manage_tags"
 )
 
 func checkProxyVal(v string) (string, error) {
@@ -418,6 +451,14 @@ func checkProxyVal(v string) (string, error) {
 	return v, nil
 }
 
+func checkCNAMEFlattenVal(v string) (string, error) {
+	v = strings.ToLower(v)
+	if v != "on" && v != "off" {
+		return "", fmt.Errorf("bad metadata value for cloudflare_cname_flatten: '%s'. Use on/off", v)
+	}
+	return v, nil
+}
+
 func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 
 	for _, rec := range dc.Records {
@@ -483,6 +524,43 @@ func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 			}
 		}
 
+		// Validate CNAME flattening metadata (only valid on CNAME records)
+		if val := rec.Metadata[metaCNAMEFlatten]; val != "" {
+			if rec.Type != "CNAME" {
+				return fmt.Errorf("cloudflare_cname_flatten set on %v record: %#v (only valid on CNAME records)", rec.Type, rec.GetLabel())
+			}
+			val, err := checkCNAMEFlattenVal(val)
+			if err != nil {
+				return err
+			}
+			rec.Metadata[metaCNAMEFlatten] = val
+		}
+
+		// Validate tags (check for reserved cf- prefix)
+		if tags := rec.Metadata[metaTags]; tags != "" {
+			for _, tag := range strings.Split(tags, ",") {
+				if strings.HasPrefix(strings.ToLower(tag), "cf-") {
+					return fmt.Errorf("cloudflare_tags on %v record %#v contains reserved tag prefix 'cf-': %q", rec.Type, rec.GetLabel(), tag)
+				}
+			}
+		}
+
+		// Validate that CF_COMMENT is only used when CF_MANAGE_COMMENTS is enabled
+		if rec.Metadata[metaComment] != "" && dc.Metadata[metaManageComments] != "true" {
+			return fmt.Errorf("CF_COMMENT used on %v record %#v but CF_MANAGE_COMMENTS is not enabled for this domain", rec.Type, rec.GetLabel())
+		}
+
+		// Validate that CF_TAGS is only used when CF_MANAGE_TAGS is enabled
+		if rec.Metadata[metaTags] != "" && dc.Metadata[metaManageTags] != "true" {
+			return fmt.Errorf("CF_TAGS used on %v record %#v but CF_MANAGE_TAGS is not enabled for this domain", rec.Type, rec.GetLabel())
+		}
+
+		// CNAME flattening and proxy are mutually exclusive (Opinion 6: if ambiguous, forbid it)
+		// Cloudflare silently disables flattening when proxy is enabled, which leads to confusing behavior
+		if rec.Type == "CNAME" && rec.Metadata[metaCNAMEFlatten] == "on" && rec.Metadata[metaProxy] == "on" {
+			return fmt.Errorf("CNAME record %#v has both CF_PROXY_ON and CF_CNAME_FLATTEN_ON set, but these are mutually exclusive; Cloudflare ignores CNAME flattening when proxy is enabled", rec.GetLabel())
+		}
+
 		if rec.Type == "CLOUDFLAREAPI_SINGLE_REDIRECT" {
 			// SINGLEREDIRECT record types. Verify they are enabled.
 			if !c.manageSingleRedirects {
@@ -509,8 +587,8 @@ func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 		if rec.Metadata[metaProxy] != "full" {
 			continue
 		}
-		ip := net.ParseIP(rec.GetTargetField())
-		if ip == nil {
+		ip, err := netip.ParseAddr(rec.GetTargetField())
+		if err != nil {
 			return fmt.Errorf("%s is not a valid ip address", rec.GetTargetField())
 		}
 		newIP, err := transform.IP(ip, c.ipConversions)
@@ -784,6 +862,23 @@ func (c *cloudflareProvider) nativeToRecord(domain string, cr cloudflare.DNSReco
 		}
 	}
 
+	// Check for CNAME flattening setting
+	if cr.Type == "CNAME" {
+		if cr.Settings.FlattenCNAME != nil && *cr.Settings.FlattenCNAME {
+			rc.Metadata[metaCNAMEFlatten] = "on"
+		} else {
+			rc.Metadata[metaCNAMEFlatten] = "off"
+		}
+	}
+
+	// Read comment and tags from API response
+	if cr.Comment != "" {
+		rc.Metadata[metaComment] = cr.Comment
+	}
+	if len(cr.Tags) > 0 {
+		rc.Metadata[metaTags] = strings.Join(cr.Tags, ",")
+	}
+
 	switch rType := cr.Type; rType { // #rtype_variations
 	case "MX":
 		if err := rc.SetTargetMX(*cr.Priority, cr.Content); err != nil {
